local AddonName, a = ...
if a.BuildFail(50000) then return end
local L = a.Localize
local s = SpellFlashAddon
local c = BittensSpellFlashLibrary
local bcm = a.BCM

local GetItemCount = GetItemCount
local IsMounted = IsMounted
local UnitBuff = UnitBuff

c.Init(a)

local function inSoloMode()
	return not s.InRaidOrParty() and not a.GetConfig("solo_off")
end

------------------------------------------------------------------------ Common
c.AddOptionalSpell("Mage Armor", nil, {
	Buff = "Mage Armor",
	BuffUnit = "player",
})

c.AddOptionalSpell("Conjure Mana Gem", nil, {
	NotIfActive = true,
	CheckFirst = function()
		local count, target
		if c.HasGlyph("Mana Gem") then
			count = GetItemCount(c.GetID("Brilliant Mana Gem"), false, true)
			target = 7
		else
			count = GetItemCount(c.GetID("Mana Gem"), false, true)
			target = 3
		end 
		if inSoloMode() then
			target = 1
		end
		return count < target
	end	
})

c.AddOptionalSpell("Mana Gem", nil, {
	Type = "item",
	CheckFirst = function()
		return a.ManaPercent < 80
	end
})

c.AddOptionalSpell("Brilliant Mana Gem", nil, {
	Type = "item",
	CheckFirst = function()
		return a.ManaPercent < 80
	end
})

c.AddOptionalSpell("Mirror Image")

c.AddOptionalSpell("Arcane Brilliance", nil, {
	NoRangeCheck = 1,
	CheckFirst = function()
		return c.RaidBuffNeeded(c.SPELL_POWER_BUFFS)
			or c.RaidBuffNeeded(c.CRIT_BUFFS)
	end
})

c.AddOptionalSpell("Dalaran Brilliance", nil, {
	NoRangeCheck = 1,
	CheckFirst = function()
		return c.RaidBuffNeeded(c.SPELL_POWER_BUFFS)
			or c.RaidBuffNeeded(c.CRIT_BUFFS)
	end
})

c.AddInterrupt("Counterspell")

c.AddOptionalSpell("Spellsteal", nil, {
	FlashColor = "aqua",
	CheckFirst = function()
		local unit = s.UnitSelection()
		if unit == nil or not s.Enemy(unit) then
			return false
		end
		
		for i = 1, 10000 do
			local _, _, _, _, _, _, _, _, isStealable, _, spellID
				= UnitBuff(unit, i)
			if spellID == nil then
				return false
			elseif isStealable then
				return true
			end
		end
	end
})

c.AddSpell("Nether Tempest", nil, {
	EarlyRefresh = 99,
	CheckFirst = function(z)
		if inSoloMode() then
			z.FlashColor = "yellow"
			z.Continue = true
		else
			z.FlashColor = nil
			z.Continue = nil
		end
		return c.GetMyDebuffDuration("Nether Tempest") < z.EarlyRefresh
	end
})
c.ManageDotRefresh("Nether Tempest", 1)

c.AddSpell("Living Bomb", nil, {
	EarlyRefresh = 99,
	CheckFirst = function(z)
		if inSoloMode() then
			z.FlashColor = "yellow"
			z.Continue = true
		else
			z.FlashColor = nil
			z.Continue = nil
		end
		return c.GetMyDebuffDuration("Living Bomb") < z.EarlyRefresh
	end
})
c.ManageDotRefresh("Living Bomb", 3)

c.AddSpell("Frost Bomb", nil, {
	CheckFirst = function(z)
		if inSoloMode() then
			z.FlashColor = "yellow"
			z.Continue = true
		else
			z.FlashColor = nil
			z.Continue = nil
		end
		return not c.IsCasting("Frost Bomb")
	end
})

c.AddOptionalSpell("Presence of Mind", nil, {
	NoGCD = true,
	CheckFirst = function()
		return not c.HasBuff("Alter Time", true)
	end
})

c.AddOptionalSpell("Evocation", nil, {
	CheckFirst = function()
		return a.ManaPercent <= 10
	end
})

c.AddOptionalSpell("Evocation", "for Health", {
	CheckFirst = function()
		return inSoloMode()
			and c.HasGlyph("Evocation")
			and s.HealthPercent("player") < 60
	end
})

c.AddOptionalSpell("Evocation", "for Invoker's Energy", {
	CheckFirst = function()
		return c.HasTalent("Invocation")
			and c.GetBuffDuration("Invoker's Energy") < c.GetHastedTime(2.25)
	end
})

c.AddOptionalSpell("Rune of Power", nil, {
	Buff = "Rune of Power",
	BuffUnit = "player",
	NoRangeCheck = true,
})

------------------------------------------------------------------------ Arcane
local function hasT6TalentFor(duration)
	return (not c.HasTalent("Rune of Power") 
			or c.GetBuffDuration("Rune of Power") >= duration)
		and (not c.HasTalent("Invocation")
			or c.GetBuffDuration("Invoker's Energy") >= duration)
end

c.AddOptionalSpell("Arcane Power", nil, {
	CheckFirst = function()
		return not c.HasBuff("Alter Time")
			and a.ChargeStacks > 1
			and a.ManaPercent > 85
			and hasT6TalentFor(15)
	end
})

c.AddOptionalSpell("Alter Time", "for Arcane", {
	CheckFirst = function()
		return not c.HasBuff("Alter Time")
			and c.HasBuff("Arcane Power")
			and c.GetBuffStack("Arcane Missiles!") == 2
			and a.ChargeStacks > 3
			and hasT6TalentFor(6)
	end
})

c.AddOptionalSpell("Evocation", "for Arcane", {
	CheckFirst = function()
		if s.Buff(c.GetID("Alter Time"), "player") then
			return false
		end
		
		if c.HasTalent("Invocation") then
			return a.ManaPercent < 25
				or c.GetBuffDuration("Invoker's Energy") 
					< c.GetCastTime("Arcane Blast") + .1
		else
			return a.ManaPercent < 60
		end
	end
})

c.AddOptionalSpell("Evocation", "Interrupt", {
	FlashColor = "red",
	Override = function()
		return c.IsCasting("Evocation")
			and not c.HasTalent("Invocation") 
			and s.PowerPercent("player") > 85
	end
})

c.RegisterForFullChannels("Arcane Missiles", 2)

c.AddSpell("Arcane Missiles", nil, {
	CheckFirst = function()
		return c.GetCooldown("Alter Time") > 4
			or not s.Flashable(c.GetID("Alter Time"))
	end
})

c.AddSpell("Arcane Missiles", "prime", {
	CheckFirst = function()
		return c.GetBuffStack("Arcane Missiles!") == 2
			or c.HasBuff("Alter Time")
			or inSoloMode()
	end
})

c.AddSpell("Arcane Blast", "prime", {
	CheckFirst = function()
		local name = s.ItemName(c.GetID("Mana Gem"))
		if s.Flashable(name)
			and GetItemCount(name, false, true) > 0
			and s.ItemCooldown(name) < 9 then
			
			return true
		end
		
		name = s.ItemName(c.GetID("Brilliant Mana Gem"))
		if s.Flashable(name)
			and GetItemCount(name, false, true) > 0
			and s.ItemCooldown(name) < 9 then
			
			return true
		end
		
		if c.HasTalent("Rune of Power") then
			return false
		end
		
		name = s.SpellName(c.GetID("Evocation"))
		return s.Flashable(name) and s.SpellCooldown(name) < 20
	end
})

c.AddSpell("Arcane Blast", "double prime", {
	CheckFirst = function()
		return a.ManaPercent > 92
			or c.HasBuff("Alter Time")
			or a.ChargeStacks < 4
	end
})

c.AddSpell("Scorch", "for Arcane", {
	CheckFirst = function()
		return not inSoloMode()
	end
})

c.AddSpell("Arcane Barrage", nil, {
	NoStopChannel = true,
	CheckFirst = function()
		return a.ManaPercent < 85
	end
})

c.AddSpell("Arcane Barrage", "if Losing Stacks", {
	FlashColor = "red",
	NoStopChannel = true,
	CheckFirst = function()
		local duration = a.ChargeDuration
		if c.HasBuff("Alter Time") then
			duration = math.min(duration, c.GetBuffDuration("Alter Time"))
		end
		return duration > 0 and duration < c.GetCastTime("Arcane Blast") + .1
	end
})

c.AddSpell("Arcane Barrage", "at 6", {
	NoStopChannel = true,
	CheckFirst = function()
		return a.ChargeStacks == 6
	end
})

c.AddSpell("Fire Blast", nil, {
	CheckFirst = function()
		return c.HasGlyph("Fire Blast") 
			and (c.GetMyDebuffDuration("Living Bomb") > 9
				or c.IsCastingOrInAir("Living Bomb")
				or c.HasMyDebuff("Nether Tempest")
				or c.IsCastingOrInAir("Nether Tempest"))
	end
})

c.AddOptionalSpell("Flamestrike", nil, {
	NoRangeCheck = true,
	CheckFirst = function()
		return not c.IsCasting("Flamestrike")
	end
})

c.AddOptionalSpell("Arcane Explosion", nil, {
	Melee = true,
})

-------------------------------------------------------------------------- Fire
c.AssociateTravelTimes(.5, "Fireball", "Pyroblast", "Frostfire Bolt")
c.AssociateTravelTimes(.2, "Scorch", "Inferno Blast")

local function combustionCheckFirst(z, multiplier)
	local threshhold = a.GetConfig("combust_at") * multiplier
	if bcm.PredictDamage(false, false) >= threshhold then
		if bcm.PredictDamage(true, false) >= threshhold then
			z.FlashColor = "yellow"
		else
			z.FlashColor = "red"
		end
		return true
	end
end

c.AddOptionalSpell("Molten Armor", nil, {
	Buff = "Molten Armor",
	BuffUnit = "player",
})

c.AddOptionalSpell("Combustion", nil, {
	NoGCD = true,
	CheckFirst = function(z)
		return combustionCheckFirst(z, 1)
	end
})

c.AddOptionalSpell("Combustion", "when Big", {
	NoGCD = true,
	CheckFirst = function(z)
		return combustionCheckFirst(z, 1.4)
	end
})

c.AddSpell("Pyroblast", nil, {
	CheckFirst = function()
		return a.PyroProc 
			or c.HasBuff("Presence of Mind") 
			or c.IsCasting("Presence of Mind")
	end
})

c.AddSpell("Inferno Blast", nil, {
	CheckFirst = function()
		return a.HeatingProc
	end
})

c.AddSpell("Scorch", nil, {
	Continue = true,
	CheckFirst = function()
		return s.Moving("player")
	end
})

c.AddSpell("Frostfire Bolt", nil, {
	Continue = true,
	CheckFirst = function()
		return c.HasGlyph("Frostfire Bolt")
	end
})

------------------------------------------------------------------------- Frost
c.AssociateTravelTimes(.7, "Frostbolt")

c.AddOptionalSpell("Summon Water Elemental", nil, {
    FlashColor = "yellow",
    CheckFirst = function()
        return not s.UpdatedVariables.PetAlive
            and not IsMounted()
    end
})

c.AddOptionalSpell("Frost Armor", nil, {
	Buff = "Frost Armor",
	BuffUnit = "player",
})

c.AddOptionalSpell("Freeze", nil, {
    NoRangeCheck = 1,
    NoGCD = true,
    CheckFirst = function()
        return a.FingerCount < 2
    end
})

c.AddOptionalSpell("Freeze", "on Pet Bar", {
    Type = "pet",
    NoRangeCheck = 1,
    NoGCD = true,
    CheckFirst = function()
        return a.FingerCount < 2
    end
})

c.AddSpell("Ice Lance", nil, {
	CheckFirst = function()
		return a.FingerCount > 0
	end
})

c.AddSpell("Ice Lance", "within 2", {
	CheckFirst = function()
		return a.FingerCount > 0 and c.GetBuffDuration("Fingers of Frost") < 2
	end
})

c.AddSpell("Ice Lance", "within 5", {
	CheckFirst = function()
		return a.FingerCount > 0 and c.GetBuffDuration("Fingers of Frost") < 5
	end
})

c.AddOptionalSpell("Frozen Orb", nil, {
	NoRangeCheck = true,
})

--c.AddOptionalSpell("Frozen Orb", "with Icy Veins", {
--	NoRangeCheck = true,
--	CheckFirst = function()
--		return a.FingerCount < 2
--			and c.HasGlyph("Icy Veins")
--			and c.GetCooldown("Icy Veins") < c.GetHastedTime(1.5)
--	end
--})

c.AddOptionalSpell("Icy Veins", nil, {
	CheckFirst = function(z)
		if c.HasGlyph("Icy Veins") then
			z.ID = c.GetID("Icy Veins Glyphed")
			return c.GetCooldown("Frozen Orb") > 50
		else
			z.ID = c.GetID("Icy Veins")
			return true
		end
	end
})

c.AddSpell("Frostbolt", "for Debuff", {
	CheckFirst = function()
		return c.GetMyDebuffStack("Frostbolt", false, true)
				+ c.CountLandings("Frostbolt", -3, 10)
			< 3
	end
})

c.AddSpell("Frostfire Bolt", "under Brain Freeze", {
	CheckFirst = function()
		return c.HasBuff("Brain Freeze")
	end
})
